#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <libserialport.h>
#include "../src/monocoque/simulatorapi/simapi/simapi/simdata.h"
#include "../src/arduino/simhaptic/simhaptic.h"

/* Helper function for error handling. */
int check(enum sp_return result);

int main(int argc, char **argv)
{

    char* p;
    int motor = strtol(argv[1], &p, 10);

    int effectspeed = strtol(argv[2], &p, 10);

    SimHapticData* simhapticdata = malloc(sizeof(SimHapticData));
    if (motor == 0 || motor == 4 || motor == 7 || motor == 8 || motor == 10 || motor == 11 || motor == 13 || motor == 14)
    {
        simhapticdata->effect1 = effectspeed;
        simhapticdata->motor1 = 1;
    }
    if (motor == 2 || motor == 6 || motor == 8 || motor == 9 || motor == 10 || motor == 11 || motor == 12 || motor == 14)
    {
        simhapticdata->effect3 = effectspeed;
        simhapticdata->motor3 = 3;
    }

    size_t size = sizeof(SimHapticData);

    char* port_name = "/dev/ttyUSB0";

    /* The ports we will use. */
    struct sp_port* port;

    /* Open and configure each port. */
    printf("Looking for port %s.\n", port_name);
    check(sp_get_port_by_name(port_name, &port));

    printf("Opening port.\n");
    check(sp_open(port, SP_MODE_READ_WRITE));

    printf("Setting port to 9600 8N1, no flow control.\n");
    check(sp_set_baudrate(port, 9600));
    check(sp_set_bits(port, 8));
    check(sp_set_parity(port, SP_PARITY_NONE));
    check(sp_set_stopbits(port, 1));
    check(sp_set_flowcontrol(port, SP_FLOWCONTROL_NONE));


        unsigned int timeout = 2000;

        /* On success, sp_blocking_write() and sp_blocking_read()
         * return the number of bytes sent/received before the
        * timeout expired. We'll store that result here. */
        int result;

        /* Send data. */
        result = check(sp_blocking_write(port, simhapticdata, size, timeout));

        /* Check whether we sent all of the data. */
        if (result == size)
        {
            printf("Sent %d bytes successfully, speed %i on motor %i.\n", size, simhapticdata->effect1, simhapticdata->motor1);
            printf("Sent %d bytes successfully, speed %i on motor %i.\n", size, simhapticdata->effect3, simhapticdata->motor3);
        }
        else
        {
            printf("Timed out, %d/%d bytes sent.\n", result, size);
        }


    check(sp_close(port));
}

/* Helper function for error handling. */
int check(enum sp_return result)
{
    /* For this example we'll just exit on any error by calling abort(). */
    char* error_message;

    switch (result)
    {
        case SP_ERR_ARG:
            printf("Error: Invalid argument.\n");
            abort();
        case SP_ERR_FAIL:
            error_message = sp_last_error_message();
            printf("Error: Failed: %s\n", error_message);
            sp_free_error_message(error_message);
            abort();
        case SP_ERR_SUPP:
            printf("Error: Not supported.\n");
            abort();
        case SP_ERR_MEM:
            printf("Error: Couldn't allocate memory.\n");
            abort();
        case SP_OK:
        default:
            return result;
    }
}
